import logging
from dataclasses import dataclass
from dataclasses import field
from dataclasses import InitVar

import myvariant

from snprimer.snp import SNP


@dataclass
class PositionRange:
    chr: str
    start: int
    end: int
    strand: str = None
    init_snp: InitVar[bool] = True
    snp: list[SNP] = field(default_factory=list)

    def __post_init__(self, init_snp):
        if init_snp:
            self.search_snp()
        else:
            logging.info(f"{self} is not initialised")

    def search_snp(self) -> list[SNP]:
        for hit in myvariant.MyVariantInfo().query(f"{self.chr}:{self.start}-{self.end}", fields="dbsnp")["hits"]:
            if (
                "dbsnp" in hit.keys() and "alleles" in hit["dbsnp"]
            ):  # SNP with no alleles informations are not processed
                self.snp.append(SNP(hit["_id"], hit["dbsnp"]["rsid"], hit["dbsnp"]["alleles"], hit["dbsnp"]["ref"]))
        return self.snp

    def get_snp(self, max_vaf: float = 0.1) -> list[SNP]:
        return [snp for snp in self.snp if snp.vaf >= max_vaf]


if __name__ == "__main__":
    a = PositionRange("chr8", 19818430, 19818440)
    for snp in a.get_snp(max_vaf=0.05):
        print(snp)



from pathlib import Path

from pyfaidx import Fasta
from pyfaidx import Sequence

from snprimer import PositionRange
from snprimer import Primer


class PrimerPair:
    def __init__(self, forward_primer: Primer, reverse_primer: Primer, compatibility: dict = None):
        self.forward_primer = forward_primer
        self.reverse_primer = reverse_primer
        self.compatibility = compatibility or {}
        self._check_compatibility()

    def _check_compatibility(self):
        if self.forward_primer.ref_version != self.reverse_primer.ref_version:
            raise Exception("Primers in a primer pair should have the same genome of reference.")

    def _make_mate_pair(self) -> list[tuple[PositionRange, PositionRange]]:
        mate_pairs = []
        for f_position_range in self.forward_primer.position_ranges:
            for r_position_range in self.reverse_primer.position_ranges:
                if self._assert_condition_for_mate_pair(f_position_range, r_position_range):
                    mate_pairs.append((f_position_range, r_position_range))
        return mate_pairs

    def _assert_condition_for_mate_pair(self, f_mate: PositionRange, r_mate: PositionRange) -> bool:
        #  TODO a lot of verification but need to read papers (max len, order -> reverse complement?)
        return (f_mate.chr == r_mate.chr) and (
            r_mate.start - f_mate.end < self.compatibility.get("max_length", 150)
        )  # TODO take from primer design

    def make_pcr(self, ref_fasta_file: Path) -> list[Sequence]:
        hits = []
        fasta_handler = Fasta(ref_fasta_file)
        for mate_pair in self._make_mate_pair():
            hits.append(
                fasta_handler[mate_pair[0].chr][mate_pair[0].start - 1 : mate_pair[1].end]
            )  # -1 because of difference of index
        return hits


if __name__ == "__main__":
    a = Primer("GGAGATGTACAGCGTGCCATAC", "hg19")
    b = Primer("TACATCTTGCTGAGGGGAAGGC", "hg19")
    pp = PrimerPair(a, b)
    print(pp.make_pcr(Path("/home/benoit/Projects/SNPrimer/tests/ref/hg19.fasta")))
    print(a)
    print(b)
